@@ -1,18 +1,8 @@
 #!/bin/python3
 
-"""
-About this Class
-
-Subject inherits all functions and attributes from the BIDSPointer class defined in this package. 
-Building on this framework, Subject is optimized to run basic first-level GLMs out of the box, 
-and users have the option to define their own design matrices as well.
-
-Ian Richard Ferguson | Stanford University
-"""
-
 # ---- Imports
-from glm_express.subject.bids_pointer import BIDSPointer
-import os, json, pathlib
+from glm_express.subject.bids_pointer import Build_Subject
+import os, json
 from time import sleep
 import pandas as pd
 import numpy as np
@@ -22,100 +12,33 @@ from nilearn.glm import first_level
 from nilearn.reporting import make_glm_report
 
 # ---- Object definition
-class Subject(BIDSPointer):
+class Subject(Build_Subject):
+      """
+      Subject inherits all functions and attributes from the BIDSPointer class defined in this package. 
+      Building on this framework, Subject is optimized to run basic first-level GLMs out of the box, 
+      and users have the option to define their own design matrices as well.
+      """
 
       # ---- Class functions
       def __init__(self, sub_id, task, bids_root, suppress=False, template_space="MNI152NLin2009",
                   repetition_time=1., dummy_scans=0):
 
             # Inherits constructor from BIDSPointer
-            BIDSPointer.__init__(self, sub_id=sub_id, task=task, bids_root=bids_root, suppress=suppress,
-                                template_space=template_space, repetition_time=repetition_time,
-                                dummy_scans=dummy_scans)
-
-            # Boolean - are modulators provided or not?
-            self.has_modulators = self._has_modulators()
-
-            if self.has_modulators:
-
-                  # Read in modulators from task_information.json file
-                  self.modulators = self._parse_modulators()
-                  
-                  # Get mean values for each modulator
-                  self._center_modulators()
-
-
-      # ---- Modeling helpers
-      def _parse_modulators(self):
-            """
-            Reads in modulators from task_information.json
-
-            Returns
-                  List of user provided modulators
-            """
-
-            with open('./task_information.json') as incoming:
-                  data = json.load(incoming)[self.task]
-
-            return list(data['modulators'])
-
-
-      def _has_modulators(self):
-            """
-            Returns
-                  Boolean | if True, modulators were provided
-            """
-
-            with open('./task_information.json') as incoming:
-                  data = json.load(incoming)[self.task]
-
-            return len(data['modulators']) > 1
-
-
-      def _center_modulators(self):
-            """
-            Calculates average values from modulators, to use for centering in design stage
-            """
-
-            # Read in all events
-            all_events = self.load_events()
-
-            # Modulators key = empty dictionary to append into
-            self.bids_container['modulators'] = {}
-
-            # Loop through user-provided modulators
-            for mod in self.modulators:
-                  
-                  """
-                  NOTE: We assume that all modulators are contained in your events file!
-
-                  If your modulators are not in the events file, they'll be removed
-                  from the Subject object
-                  """
-                  
-                  if mod not in all_events.columns:
-                        print(f'\n** ERROR: {mod.upper()} not in Events File\n')
-                        self.modulators.remove(mod)
-                        continue
-
-                  # Mean value from modulator column
-                  avg = all_events[mod].mean()
-
-                  # Assign to bids_container object
-                  self.bids_container['modulators'][mod] = avg
+            Build_Subject.__init__(self, sub_id=sub_id, task=task, bids_root=bids_root, suppress=suppress,
+                                   template_space=template_space, repetition_time=repetition_time,
+                                   dummy_scans=dummy_scans)
 
 
 
       # ---- Modeling functions
-      def generate_design_matrix(self, run, non_steady_state=False, include_modulators=False, 
-                                 auto_block_regressors=False, motion_outliers=True, drop_fixation=True):
+      def generate_design_matrix(self, run, non_steady_state=False, auto_block_regressors=False, 
+                                 motion_outliers=True, drop_fixation=True, a_comp_cor=True, t_comp_cor=True):
             """
             Builds a first level design matrix
 
             Parameters
                   run: int | Functional run number, lines up with events file
                   non_steady_state: boolean | if True, all non steady state regressors are aggregated as one regressor
-                  include_modulators: boolean | if True, Parametric modulators are calculated and included in design matrix
                   auto_block_regressors: boolean | if True, `block_type` and `trial_type` columns are implicitly merged
                   motion_outliers: boolean | if True, all motion outliers are included in design matrix
                   drop_fixation: boolean | if True, fixation trials are dropped from event files
@@ -165,39 +88,6 @@ class Subject(BIDSPointer):
                   return f'{block}_{trial}'
 
 
-            def create_temp_dataframe(events, mod_name, frame_times):
-                  """
-                  Creates temporary DataFrame to merge into events DF (optimized to handle modulators)
-
-                  Parameters
-                        events: Pandas DataFrame |
-                        mod_name: str |
-                        frame_times: int |
-
-                  Returns
-                        Reduced Pandas DataFrame
-                  """
-
-                  # Nilearn implicitly requires a modulation column - we'll rename the column of interest here
-                  temp = events.rename(columns={mod_name: 'modulation'})
-
-                  # Calculate distance from the mean for the current modulation
-                  temp['modulation'] = temp['modulation'] - self.bids_container['modulators'][mod_name]
-
-                  # Rename trial_type => trial_type_x_{modulation}
-                  temp['trial_type'] = temp['trial_type'].apply(lambda x: f'{x}_x_{mod_name}')
-
-                  # These will be the only columns we output
-                  output_columns = list(temp['trial_type'].unique())
-
-                  # Translate DataFrame to Design Matrix
-                  dm = first_level.make_first_level_design_matrix(frame_times,
-                                                                  temp,
-                                                                  hrf_model='spm')
-
-                  # Return isolated trial x modulation columns AND index
-                  return dm.loc[:, output_columns].reset_index()
-
 
             def reorder_design_columns(DM):
                   """
@@ -232,16 +122,13 @@ class Subject(BIDSPointer):
             # === Model foundations ===
             iso_container = self.bids_container[f'run-{run}']                                   # BIDS container for the current run
             events = pd.read_csv(iso_container['event'], sep='\t')                              # Load events file
-            confounds = pd.read_csv(iso_container['confound'], sep='\t')                        # Load fmriprep regressors
+            confounds = pd.read_csv(iso_container['confounds'], sep='\t')                       # Load fmriprep regressors
             voi = ['onset', 'duration', 'trial_type']                                           # Starting variables of interest
 
             if auto_block_regressors:
                   if 'block_type' in events.columns:
                         voi += ['block_type']                                                   # Add block type to VOI if True
 
-            if include_modulators:
-                  voi += self.modulators                                                        # Add modulators to VOI if True
-
             events = events.loc[:, voi]                                                         # Reduce 
             
 
@@ -260,14 +147,6 @@ class Subject(BIDSPointer):
                   events['trial_type'] = events.apply(block_regressor, axis=1)
                   events.drop(columns=['block_type'], inplace=True)
 
-            # Create mod-wise Design Matrices if True
-            if include_modulators:
-                  mod_matrices = []
-
-                  for novel_modulator in self.modulators:
-                        mod_matrices.append(create_temp_dataframe(events, novel_modulator, frame_times))
-
-
             # Set object conditions to match trial types
             clean_conditions = list(events['trial_type'].dropna().unique())
             self.set_conditions(clean_conditions)
@@ -275,11 +154,6 @@ class Subject(BIDSPointer):
             # Create baseline Design Matrix with defined parameters
             events = first_level.make_first_level_design_matrix(frame_times, events, hrf_model='spm').reset_index()
 
-            # Merge modulated DataFrames into Design Matrix if True
-            if include_modulators:
-                  for matrix in mod_matrices:
-                        events = events.merge(matrix, on='index', how='left')
-
             # We assume there are no dummy scans unless otherwise specified
             try:
                   dummy_value = iso_container['dummy']
@@ -298,6 +172,14 @@ class Subject(BIDSPointer):
             if motion_outliers:
                   confound_vars += [x for x in list(confounds.columns) if 'motion_outlier' in x]
 
+            # Pull in anatomical noise component derivatives if True
+            if a_comp_cor:
+                  confound_vars += [x for x in list(confounds.columns) if "a_comp_cor" in x]
+
+            # Pull in temporal noise component derivatives if True
+            if t_comp_cor:
+                  confound_vars += [x for x in list(confounds.columns) if "t_comp_cor" in x]
+
             # Isolate confound regressor
             confounds = confounds.loc[:, confound_vars].reset_index()
 
@@ -326,14 +208,14 @@ class Subject(BIDSPointer):
 
 
 
-      def first_level_design(self, non_steady_state=False, include_modulators=False, auto_block_regressors=False,
-                            motion_outliers=True, drop_fixation=True, verbose=True):
+      def first_level_design(self, non_steady_state=False, auto_block_regressors=False,
+                            motion_outliers=True, drop_fixation=True, verbose=True, a_comp_cor=True,
+                            t_comp_cor=True):
             """
             Compiles one design matrix per functional run
 
             Parameters
                   non_steady_state: boolean | if True, all non steady state regressors are aggregated as one regressor
-                  include_modulators: boolean | if True, Parametric modulators are calculated and included in design matrix
                   auto_block_regressors: boolean | if True, `block_type` and `trial_type` columns are implicitly merged
                   motion_outliers: boolean | if True, all motion outliers are included in design matrix
                   drop_fixation: boolean | if True, fixation trials are dropped from event files
@@ -342,15 +224,14 @@ class Subject(BIDSPointer):
                   List of Pandas DataFrame objects (one per functional run)
             """
 
-            model_specs = f"""
-            \n\nRunning first-level designs for {self.task.upper()} with the following parameters:\n\n
-
-            Non-steady state regressors:\t{non_steady_state}\n
-            Modulators:\t\t\t\t{include_modulators}\n
-            Auto-block regressors:\t\t{auto_block_regressors}\n
-            Motion outliers:\t\t\t{motion_outliers}\n
-            Fixation trials:\t\t\t{drop_fixation}
-            \n\n
+            model_specs = f"""\n\nRunning first-level designs for {self.task.upper()} with the following parameters:\n\n
+Non-steady state regressors:\t\t{non_steady_state}\n
+Auto-block regressors:\t\t\t{auto_block_regressors}\n
+Motion outliers:\t\t\t{motion_outliers}\n
+Fixation trials:\t\t\t{drop_fixation}\n
+Anatomical noise components:\t\t{a_comp_cor}\n
+Temporal noise components:\t\t{t_comp_cor}
+\n\n
             """
             
             if verbose:
@@ -367,8 +248,12 @@ class Subject(BIDSPointer):
                   run = k+1
 
                   # Generate design matrix given the input parameters
-                  matrix = self.generate_design_matrix(run, non_steady_state, include_modulators,
-                                                      auto_block_regressors, motion_outliers, drop_fixation)
+                  matrix = self.generate_design_matrix(run, non_steady_state=non_steady_state,
+                                                       auto_block_regressors=auto_block_regressors, 
+                                                       motion_outliers=motion_outliers, 
+                                                       drop_fixation=drop_fixation,
+                                                       a_comp_cor=a_comp_cor,
+                                                       t_comp_cor=t_comp_cor)
 
                   # Output filename
                   filename = f'sub-{self.sub_id}_task-{self.task}_run-{run}_design-matrix.jpg'
@@ -402,15 +287,15 @@ class Subject(BIDSPointer):
                         if outer != inner:
 
                               # Avoid duplicates - these can be modeled using directionality
-                              if (f'{inner} - {outer}') not in list(contrasts.keys()):
-                                    k = f'{outer} - {inner}'
+                              if (f'{inner}-{outer}') not in list(contrasts.keys()):
+                                    k = f'{outer}-{inner}'
                                     contrasts[k] = k
 
             return contrasts
 
 
 
-      def _run_contrast(self, glm, contrast, title, output_type, smoothing, plot_brains=False):
+      def _run_contrast(self, glm, contrast, title, output_type, smoothing, plot_brains=False, plot_type="stat"):
             """
             Linear contrast based on first-level GLM. NifTi output is always saved, stat maps are conditional
 
@@ -456,12 +341,19 @@ class Subject(BIDSPointer):
 
                   # Formatted output paths
                   stat_output = f'{plot_base}/sub-{self.sub_id}_{output_type}-{title}_smoothing-{kernel}mm_plot-stat-map.png'
+                  glass_output = f'{plot_base}/sub-{self.sub_id}_{output_type}-{title}_smoothing-{kernel}mm_plot-glass-map.png'
                   report_output = f'{plot_base}/sub-{self.sub_id}_{output_type}-{title}_smoothing-{kernel}mm_contrast-summary.html'
 
                   # Plot stat map
-                  nip.plot_stat_map(z_map, threshold=2.3, colorbar=False, draw_cross=False,
-                                    display_mode='ortho', title=title,
-                                    output_file=stat_output)
+                  if plot_type == "stat":
+                        nip.plot_stat_map(z_map, threshold=2.3, colorbar=False, draw_cross=False,
+                                          display_mode='ortho', title=title,
+                                          output_file=stat_output)
+
+                  elif plot_type == "glass":
+                        nip.plot_glass_brain(z_map, threshold=2.3, plot_abs=False,
+                                             display_mode="lyrz", title=title,
+                                             output_file=glass_output)
 
                   # Make GLM report
                   make_glm_report(model=glm, contrasts=contrast,
@@ -469,9 +361,11 @@ class Subject(BIDSPointer):
 
 
 
-      def run_first_level_glm(self, conditions=True, contrasts=True, smoothing=8., plot_brains=True, user_design_matrices=None,
-                              non_steady_state=False, include_modulators=False, auto_block_regressors=False,
-                              motion_outliers=True, drop_fixation=True, verbose=True):
+      def run_first_level_glm(self, conditions=True, contrasts=True, smoothing=8., 
+                              plot_brains=True, user_design_matrices=None,
+                              non_steady_state=False, auto_block_regressors=False,
+                              motion_outliers=True, drop_fixation=True, verbose=True, 
+                              plot_type="stat", a_comp_cor=True, t_comp_cor=True):
 
             """
             Instantiates and fits a FirstLevelModel GLM object, compiles condition and contrast z-maps
@@ -483,10 +377,10 @@ class Subject(BIDSPointer):
                   plot_brains: boolean | if True, local contrast images are saved to the first level output directory
                   user_design_matrices: Defaults to None | if supplied, these replace the Object-generated design matrices
                   non_steady_state: boolean | if True, all non steady state regressors are aggregated as one regressor
-                  include_modulators: boolean | if True, Parametric modulators are calculated and included in design matrix
                   auto_block_regressors: boolean | if True, `block_type` and `trial_type` columns are implicitly merged
                   motion_outliers: boolean | if True, all motion outliers are included in design matrix
                   drop_fixation: boolean | if True, fixation trials are dropped from events file
+                  plot_type: str | stat or glass
             """
 
             # === Define DMs and contrasts ===
@@ -495,9 +389,13 @@ class Subject(BIDSPointer):
                   matrices = user_design_matrices
 
             else:
-                  matrices = self.first_level_design(non_steady_state=non_steady_state, include_modulators=include_modulators,
-                                                   auto_block_regressors=auto_block_regressors, motion_outliers=motion_outliers,
-                                                   drop_fixation=drop_fixation, verbose=verbose)
+                  matrices = self.first_level_design(non_steady_state=non_steady_state,
+                                                     auto_block_regressors=auto_block_regressors, 
+                                                     motion_outliers=motion_outliers, 
+                                                     drop_fixation=drop_fixation, 
+                                                     verbose=verbose, 
+                                                     a_comp_cor=a_comp_cor, 
+                                                     t_comp_cor=t_comp_cor)
             
             # If user has not provided updated contrast map, the default pairwsie contrasts will run
             if self.contrasts == 'default':
@@ -523,7 +421,9 @@ class Subject(BIDSPointer):
                   print('\n=== Fitting GLM ===')
 
             # Fit to functional runs and design matrices
-            model = glm.fit(self.bids_container['all_func'], design_matrices=matrices)
+            model = glm.fit(self.bids_container['all_preprocessed_bold'], 
+                            design_matrices=matrices)
+
 
             # === Conditionally run condition and contrast maps ===
             if verbose:
@@ -537,16 +437,23 @@ class Subject(BIDSPointer):
                         print('\n=== Mapping condition z-scores ===\n')
                   
                   for condition in tqdm(self.conditions, disable=disable):
-                        self._run_contrast(glm=model, contrast=condition, title=condition, output_type='condition',
-                                          smoothing=smoothing, plot_brains=plot_brains)
+                        self._run_contrast(glm=model, contrast=condition, title=condition, 
+                                           output_type='condition', smoothing=smoothing, 
+                                           plot_brains=plot_brains,
+                                           plot_type=plot_type)
 
-            if contrasts:
+            if (contrasts) and (len(self.contrasts)) > 0:
                   if verbose:
                         print('\n=== Mapping contrast z-scores ===\n')
 
                   for k in tqdm(list(contrasts_to_map.keys()), disable=disable):
-                        self._run_contrast(glm=model, contrast=contrasts_to_map[k], title=k,
-                                           output_type='contrast', smoothing=smoothing, plot_brains=plot_brains)
+                        self._run_contrast(glm=model,
+                                           contrast=contrasts_to_map[k], 
+                                           title=k,
+                                           output_type='contrast', 
+                                           smoothing=smoothing, 
+                                           plot_brains=plot_brains,
+                                           plot_type=plot_type)
 
             if verbose:
-                  print(f'\n\n=== {self.task.upper()} contrasts computed! Subject {self.sub_id} has been mapped ===\n\n') 
+                  print(f'\n\n=== {self.task.upper()} contrasts computed! Subject {self.sub_id} has been mapped ===\n\n') 
\ No newline at end of file
